﻿//*************************************************************************************************
//【サンプルプログラム】『Kinectからカラー画像を取得する』
//【名称】Bitmap_GetColorImage　　【SDK Version】1.7
//【作成者】私設研究所Neo-Tech-Lab.com  上田智章
//【作成日】2013年8月30日
//*************************************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;             //【Debug】
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Runtime.InteropServices; //【Marshal.Copy】
using Microsoft.Kinect;               //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択

namespace Bitmap_GetColorImage
{
    public partial class Form1 : Form
    {
        //【KINECT関連】
        //　関数内でダイナミックに変数を確保すると動作速度が遅くなるので、ここにまとめて変数を確保しています。
        private KinectSensor KS_KinectSensor;   //【KINECT】
        private byte[] KS_ColorImageDataXRGB;   //【KINECTのColorImageData】32bit/画素,X8R8G8B8表示形式
        private int KS_ColorFrameWidth;         //【KINECTのColorFrameの横方向画素数】
        private int KS_ColorFrameHeight;        //【KINECTのColorFrameの縦方向画素数】
        private Bitmap KS_bmp1;                 //【ColorImage表示用Bitmapオブジェクト】
        private BitmapData KS_data1;            //【ColorImage用BitmapData】
        private Rectangle KS_rect1;             //【ColorImage表示枠】
        private bool KS_flagFlip = true;        //【左右反転フラグ】

        public Form1()
        {
            InitializeComponent();

            //***********************************************************
            //【Kinect検出】【共通】
            //***********************************************************
            //【利用可能なKINECTセンサを取得する】
            KS_KinectSensor = (from sensorToCheck
                               in KinectSensor.KinectSensors 
                               where sensorToCheck.Status == KinectStatus.Connected
                               select sensorToCheck).FirstOrDefault();
            //【検出結果の表示】
            pictureBox2.SizeMode = PictureBoxSizeMode.StretchImage;
            if (KS_KinectSensor == null) //【Kinect接続が確認できなかった場合】
            {
                pictureBox2.Image = Bitmap.FromFile("../../image/KinectOff.png");
                MessageBox.Show(
                   "接続可能なKINECTセンサが見当たりません。",
                   "KINECT for Windows Error Message",
                   MessageBoxButtons.OK
                );
                return;
            }
            pictureBox2.Image = Bitmap.FromFile("../../image/KinectOn.png");

            //***********************************************************
            //【Kinectの初期化処理】
            //***********************************************************
            //【KINECTの設定】ColorStreamを許可
            KS_KinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30); //【ColorStream許可】
            //■他に.RgbResolution1280x960Fps12も有り。ColorImageFormatを参照の事。

            //【KINECT動作開始】
            KS_KinectSensor.Start(); //【KINECTスタート】

            //【KINECTデータ取得イベントの設定とデータバッファの確保】
            KS_KinectSensor.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(ColorImageReady); //【ColorImageReadyをcallするよう設定】
            KS_ColorImageDataXRGB = new byte[KS_KinectSensor.ColorStream.FramePixelDataLength]; //【カラー画像データ１フレーム分を保存するメモリを確保】
            KS_ColorFrameWidth = KS_KinectSensor.ColorStream.FrameWidth;   //【ColorImageの横方向画素数】
            KS_ColorFrameHeight = KS_KinectSensor.ColorStream.FrameHeight; //【ColorImageの縦方向画素数】
            KS_bmp1 = new Bitmap(KS_ColorFrameWidth, KS_ColorFrameHeight, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【ColorImage表示用】
            KS_rect1 = new Rectangle(0, 0, KS_ColorFrameWidth, KS_ColorFrameHeight); //【ColorImage用枠】
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage; //【キャプチャー画像がpictureBox枠内にはめ込まれます】

            //【Kinectヘッドの仰角を設定】設定範囲は±27°
            int Angle = 0;
            KS_KinectSensor.ElevationAngle = Angle;
            textBox1.Text = Angle.ToString();

            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);
        }

        //***************************************************************
        //【終了処理】
        //***************************************************************
        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (KS_KinectSensor != null)
            {
                KS_KinectSensor.Stop();    //【KINECTセンサを終了】★これを実行しないとレーザーが消えない
                KS_KinectSensor.Dispose(); //【KINECTセンサのオブジェクトを解放する】
            }
        }

        //***************************************************************
        //【ColorImageの取得イベント】
        //***************************************************************
        // Kinectからカラー画像１フレーム分が到着する度に、この関数が呼ばれます。
        // Windowsパソコンが最も高速にBitmapデータを表示できるデータ形式は1画素４バイトの形式なのでXRGB形式を使っています。
        // 但し、追加のCG描画で半透明表現を使うときはARGB形式を使います。
        private void ColorImageReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame f = e.OpenColorImageFrame())
            {
                if (f != null)
                {
                    f.CopyPixelDataTo(KS_ColorImageDataXRGB); //【KINECTのColor画像を配列PixelDataにコピー】
                    KS_data1 = KS_bmp1.LockBits(KS_rect1, ImageLockMode.WriteOnly, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【Lock】
                    Marshal.Copy(KS_ColorImageDataXRGB, 0, KS_data1.Scan0, KS_ColorImageDataXRGB.Length); //【UnmanagedMemoryのBitmapDataに配列PixelDataをコピー】
                    KS_bmp1.UnlockBits(KS_data1);           //【Lock解除】
                    if (KS_flagFlip) { KS_bmp1.RotateFlip(RotateFlipType.Rotate180FlipY); } //【左右反転】
                //  pictureBox1.BackgroundImage = KS_bmp1;  //【pictureBox1に画像表示】
                    pictureBox1.Image = KS_bmp1;            //【pictureBox1に画像表示】

                    Graphics g = Graphics.FromImage(pictureBox1.Image);
                 //   g.Clear(Color.FromArgb(0x01000000));
                    SolidBrush b = new SolidBrush(Color.Red);
                    Font ft = new Font("Times New Roman", 30);
                    Vector4 va = KS_KinectSensor.AccelerometerGetCurrentReading();
                    g.DrawString("【内蔵加速度センサ】", ft, b, new PointF(250, 320));
                    g.DrawString("X=" + va.X.ToString(), ft, b, new PointF(330, 360));
                    g.DrawString("Y="+va.Y.ToString(), ft, b, new PointF(330, 400));
                    g.DrawString("Z="+va.Z.ToString(), ft, b, new PointF(330, 440));
                    g.Dispose();
                    Validate();
                }
                else
                {
                    //【ImageFrameがnullだったのは要求が間に合わなかったと考えられます】
                }
            }
        }

        //***************************************************************
        //【上下角変更ボタン】KINECTの上下角を変更。±27°の範囲内。時間を置かないとエラーが発生する
        //***************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            Int32 angle = int.Parse(textBox1.Text); //【設定角度を読み込む】±27°
            if (angle <= KS_KinectSensor.MaxElevationAngle && angle >= KS_KinectSensor.MinElevationAngle)
            {
                KS_KinectSensor.ElevationAngle = angle; //【上下角の変更】
            }
        }

        //***************************************************************
        //【カラー画像の左右反転指定フラグ】
        //***************************************************************
        //　ここではフラグを立てるだけで、実際の処理はColorImageReady内で行います。
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked) { KS_flagFlip = true; } else { KS_flagFlip = false; }
        }
    }
}
